#line 2 "macrolexer.c"
#line 2 "macro.l"

/* ex: set ro ft=c:
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically by the Parrot build process
 * from the file compilers/pirc/macro/macro.l.
 *
 * Any changes made here will be lost!
 *
*/

/* HEADERIZER HFILE: none */
/* HEADERIZER STOP */

#ifndef __STDC_VERSION__
#  define __STDC_VERSION__ 0
#endif




#line 25 "macrolexer.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */

/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

/* %endif */

/* %if-c++-only */
/* %endif */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else   /* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif  /* defined (__STDC__) */
#endif  /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* %not-for-header */

/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE macrorestart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    /* Note: We specifically omit the test for yy_rule_can_match_eol because it requires
     *       access to the local variable yy_act. Since yyless() is a macro, it would break
     *       existing scanners that call yyless() from OUTSIDE macrolex.
     *       One obvious solution it to make yy_act a global. I tried that, and saw
     *       a 5% performance hit in a non-yylineno scanner, because yy_act is
     *       normally declared as a register variable-- so it is not worth it.
     */
    #define  YY_LESS_LINENO(n) \
            do { \
                int yyl;\
                for ( yyl = n; yyl < yyleng; ++yyl )\
                    if ( yytext[yyl] == '\n' )\
                        --yylineno;\
            }while(0)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        *yy_cp = yyg->yy_hold_char; \
        YY_RESTORE_YY_MORE_OFFSET \
        yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up yytext again */ \
        } \
    while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
    {
/* %if-c-only */
    FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

    char *yy_ch_buf;        /* input buffer */
    char *yy_buf_pos;       /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
     * characters.
     */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
     * characters.
     */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
     * and can realloc() it to grow it, and should free() it to
     * delete it.
     */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
     * if we're using stdio for input, then we want to use getc()
     * instead of fread(), to make sure we stop fetching input after
     * each newline.
     */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
     * If so, '^' rules will be active on the next match, otherwise
     * not.
     */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
     * end of it.
     */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
     * then we mark the buffer as YY_EOF_PENDING, to indicate that we
     * shouldn't try reading from the input source any more.  We might
     * still have a bunch of tokens to match, though, because of
     * possible backing-up.
     *
     * When we actually see the EOF, we change the status to "new"
     * (via macrorestart()), so that the user can continue scanning by
     * just pointing yyin at a new input file.
     */
#define YY_BUFFER_EOF_PENDING 2

    };
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

void macrorestart (FILE *input_file ,yyscan_t yyscanner );
void macro_switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void macro_delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void macro_flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void macropush_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void macropop_buffer_state (yyscan_t yyscanner );

static void macroensure_buffer_stack (yyscan_t yyscanner );
static void macro_load_buffer_state (yyscan_t yyscanner );
static void macro_init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );

#define YY_FLUSH_BUFFER macro_flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE macro_scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE macro_scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );

/* %endif */

void *macroalloc (yy_size_t ,yyscan_t yyscanner );
void *macrorealloc (void *,yy_size_t ,yyscan_t yyscanner );
void macrofree (void * ,yyscan_t yyscanner );

#define yy_new_buffer macro_create_buffer

#define yy_set_interactive(is_interactive) \
    { \
    if ( ! YY_CURRENT_BUFFER ){ \
        macroensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol) \
    { \
    if ( ! YY_CURRENT_BUFFER ){\
        macroensure_buffer_stack (yyscanner); \
        YY_CURRENT_BUFFER_LVALUE =    \
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
    } \
    YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define macrowrap(n) 1
#define YY_SKIP_YYWRAP

#define FLEX_DEBUG

typedef unsigned char YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yy_fatal_error (yyconst char msg[] ,yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
    yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
    yyleng = (size_t) (yy_cp - yy_bp); \
    yyg->yy_hold_char = *yy_cp; \
    *yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
    yyg->yy_c_buf_p = yy_cp;

/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 72
#define YY_END_OF_BUFFER 73
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
    {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
    };
static yyconst flex_int16_t yy_accept[352] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       73,   71,   69,   70,   69,   71,   71,   71,   71,   71,
       71,   26,   71,   71,   44,   71,   71,   71,   63,   63,
       71,   71,   71,   71,   59,   59,   71,   71,   71,   71,
       71,    4,   69,   71,   71,   71,   71,   71,   71,   47,
       47,   36,   36,   71,   71,    2,   69,   21,   23,   22,
        3,   69,   71,   71,   59,   59,   25,   25,    1,   25,
       25,   25,   25,   24,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   71,   19,   19,   19,   19,

       19,   31,   31,    5,   31,   30,   31,   31,   31,   31,
       31,   31,   31,   31,   31,   31,   31,   31,   31,   32,
       31,   71,   71,   71,   28,   71,   27,   71,   71,   52,
       52,   33,   33,   29,   70,   70,   70,   66,    0,   58,
        0,   42,   42,   68,    0,    0,   67,    0,   63,    0,
        0,    0,   62,   37,   37,   37,   37,   68,   62,    0,
        0,    0,    0,   60,   66,   66,   59,   65,   40,   57,
        4,    4,    4,    0,   46,    0,    0,    0,    0,   47,
       49,   49,    0,    0,    0,    0,   36,   36,    0,   50,
        0,    0,    2,    2,    2,    3,    3,    3,    0,    0,

        0,   61,   61,   61,   59,   11,   57,    1,    1,    1,
       18,   19,   19,   19,   19,   19,    5,    5,    5,    0,
       51,    0,   42,    0,    0,   52,   56,   34,   34,   34,
       34,   56,    0,    0,    0,    0,   33,   33,   33,   42,
       38,   42,    0,   43,    0,   41,   41,    0,    0,   37,
       37,   37,   37,   62,   64,    0,   62,   64,   64,   60,
       66,   39,   45,    0,   49,   48,    0,   49,   48,   48,
        0,   10,   57,   12,   61,   61,   61,   18,   14,   15,
       16,   19,   35,    0,   34,   34,   34,   34,   56,   54,
        0,   56,   55,   53,   39,   41,   41,    0,   62,   37,

       37,    0,   49,   12,   61,   61,   19,    0,   56,   34,
       34,   37,   37,   20,   61,   19,   34,   34,   37,    8,
       61,   17,   34,    8,   37,   37,   61,   34,   34,    6,
       37,   61,    6,   34,   37,   61,   61,   34,   37,   61,
       61,   34,   37,   61,   61,   34,    7,    9,   13,    7,
        0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    2,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    2,    1,    1,    1,    1,
        1,    5,    6,    7,    1,    8,    9,   10,   11,   12,
       13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
       22,   22,   22,   22,   22,   23,   23,   24,    1,   25,
       26,   27,    1,   28,   29,   30,   29,   29,   31,   29,
       32,   32,   33,   32,   32,   32,   32,   33,   34,   33,
       32,   32,   33,   32,   32,   32,   32,   35,   32,   32,
        1,   36,    1,    1,   37,    1,   38,   39,   40,   41,

       42,   29,   43,   32,   44,   32,   32,   45,   46,   47,
       48,   49,   32,   50,   51,   52,   53,   32,   32,   35,
       32,   32,   54,   55,   56,   57,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[58] =
    {   0,
        1,    1,    2,    1,    1,    1,    3,    4,    1,    1,
        3,    1,    1,    1,    1,    1,    1,    1,    1,    5,
        5,    5,    5,    6,    1,    4,    1,    7,    8,    8,
        8,    7,    7,    7,    7,    1,    7,    8,    8,    8,
        8,    8,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[391] =
    {   0,
        0,    0,   56,  112,  169,    0,   56,   57,  224,    0,
      257,    0,  313,  369,  419,    0,  471,  527,  584,    0,
     1201, 1286, 1286,   74, 1190, 1163,   62, 1146, 1142,   70,
     1149, 1286,   71,   66, 1286,   81,  635,   74,  664,  122,
        0,   65,   68,   79,   84,  126,   97,   84, 1130, 1122,
     1114,  130, 1127,  129, 1087, 1099,  137,  230,  321,  695,
      334,    0,  313,  313, 1048,  356, 1055, 1286, 1286, 1286,
      387, 1045, 1014,  389, 1022,  378, 1286, 1286,  401, 1026,
     1002, 1000,   99, 1286,  142,  992, 1286,  981,  980,  327,
       94,  135,  203,  295,  336,    0,    0,  958,  939,  945,

      935, 1286, 1286,  414,  974, 1286,  950,  949,  223,  229,
      932, 1286,  931,  929,  388,  338,  340,  394,  322, 1286,
      382,  344,  910,  930, 1286,  420, 1286,  481,  737,  766,
      492,  912,  472, 1286,  514,  545,  553, 1286,  448, 1286,
        0,  906,  539, 1286,  917,  900, 1286,  544,  629,  472,
      892,  913,  669,    0,  548,  871,  879,  890,  700,  402,
      557,  408,    0,    0,  888,  406,  210, 1286,  454,    0,
      660,  670,  705,  484, 1286,    0,  718,  902,  729,  751,
      733,  789,  416,  801,  518,    0,    0,  745,  499, 1286,
        0,  896,  774,  823,  830,  834,  838,  842,  882,  828,

        0,    0,  858,  854,  806, 1286,  833,  856,  860,  864,
        0,    0,  777,  773,  777,  765,  868,  872,  876,  515,
     1286,    0,  862,  802,  867,  873,  877,    0,  889,  760,
      768,  900,  424,  912,  656,    0,  771,  481,  916,  774,
     1286,  925,  647, 1286,    0,    0,  930,  784,  939,    0,
      943,  754,  745,  947,  504,  951,  959,  770,    0,    0,
      754, 1286,  963,  973,  977,  555,  981,  989,  782,    0,
      738, 1286,  993,    0,    0,  720,  707,    0,    0,    0,
        0,  702, 1000, 1010,    0, 1014,  696,  695, 1018,  624,
     1022, 1030,  881,    0, 1286,    0, 1034, 1041, 1045,  688,

      682, 1049, 1053,    0,  682,  677,  677, 1057, 1061,  674,
      664,  652,  656,    0,  654,  658,  644,  648,  652,  651,
      643,    0,  629,  632,  625,  626,  614,  611,  608,    0,
      533,  473,    0,  525,  436,  395,  385,  366,  318,  303,
      299,  273,  202,  121,  106,   72,    0,    0,    0,    0,
     1286, 1084, 1092, 1100, 1108, 1110, 1118, 1124, 1126, 1130,
     1138, 1146, 1150, 1158, 1166, 1168, 1173, 1177, 1179, 1183,
     1191, 1199, 1205, 1209, 1213, 1221, 1223, 1231, 1235, 1239,
     1243, 1247, 1251, 1253, 1257, 1261, 1265, 1269, 1273, 1277
    } ;

static yyconst flex_int16_t yy_def[391] =
    {   0,
      351,    1,  352,  352,  351,    5,    4,    4,    5,    9,
      351,   11,  353,  353,    4,   15,  354,  354,  351,   19,
      351,  351,  351,  351,  351,  351,  355,  356,  351,  351,
      357,  351,  351,  351,  351,  351,  358,  351,  351,  351,
      359,  351,  351,  351,  360,  360,  351,  351,  351,  351,
      351,  351,  351,  361,  351,  362,  351,  351,  351,  351,
      351,  363,  363,  364,  365,  351,  351,  351,  351,  351,
      351,  351,  366,  367,  368,  368,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  369,  370,  370,  370,  370,

      370,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  371,  356,  372,  351,  351,  351,  351,  373,  351,
      351,  374,  374,  351,  351,  351,  351,  351,  355,  351,
      355,  375,  375,  351,  357,  351,  351,  351,  351,  376,
      377,  378,  351,  379,  379,  379,  379,  351,  351,  351,
      351,  351,  380,  381,  351,  351,  360,  351,  360,   46,
      351,  351,  351,  361,  351,  361,  351,  362,  351,  351,
      351,  351,  351,  351,  351,  382,  363,  363,  364,  351,
      364,  365,  351,  351,  351,  351,  351,  351,  383,  383,

      384,  385,  385,  385,  368,  351,  368,  351,  351,  351,
      386,  370,  370,  370,  370,  370,  351,  351,  351,  371,
      351,  371,  375,  372,  351,  351,  351,  387,  387,  387,
      387,  351,  351,  351,  351,  388,  374,  374,  374,  375,
      351,  375,  376,  351,  376,  389,  389,  378,  351,  379,
      379,  379,  379,  351,  351,  351,  351,  351,  380,  381,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  382,
      383,  351,  383,  390,  385,  385,  385,  386,  370,  370,
      370,  370,  375,  351,  387,  387,  387,  387,  351,  351,
      351,  351,  351,  388,  351,  389,  389,  351,  351,  379,

      379,  351,  351,  390,  385,  385,  370,  351,  351,  387,
      387,  379,  379,  385,  385,  370,  387,  387,  379,  379,
      385,  370,  387,  387,  379,  379,  385,  387,  387,  379,
      379,  385,  387,  387,  379,  385,  385,  387,  379,  385,
      385,  387,  379,  385,  385,  387,  379,  385,  385,  387,
        0,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351
    } ;

static yyconst flex_int16_t yy_nxt[1344] =
    {   0,
       22,   23,   24,   25,   23,   26,   27,   28,   29,   30,
       31,   32,   22,   33,   34,   35,   36,   37,   38,   39,
       40,   40,   40,   41,   42,   43,   44,   45,   45,   45,
       45,   45,   46,   45,   45,   22,   45,   45,   45,   45,
       45,   45,   45,   45,   45,   45,   45,   45,   45,   45,
       45,   45,   45,   22,   47,   22,   48,   23,   24,   25,
       23,   26,   64,   64,   29,   30,   65,   65,  140,   33,
       49,   35,   50,   51,   38,  135,  135,  136,  135,  144,
       42,   43,   44,  148,  146,  149,  149,  149,  149,  165,
      138,  147,  158,  138,  138,  144,  147,  141,  148,  147,

      149,  149,  149,  149,  138,  166,  147,  168,  144,  144,
       47,  169,   48,   23,   24,   25,   23,   26,  165,  138,
       29,   30,  144,  350,  144,   33,   49,   35,   50,   51,
       38,  171,  171,  172,  171,  175,   42,   43,   44,  159,
      144,  149,  149,  149,  149,  170,  170,  170,  170,  168,
      349,  144,  161,  169,  179,  146,  180,  180,  180,  180,
      138,  138,  147,  161,  176,  348,   47,  147,   48,   22,
       23,   52,   53,   23,   26,   54,   55,   29,   30,   56,
       22,   22,   33,   57,   35,   58,   59,   38,   60,   61,
       61,   61,   22,   42,   43,   44,   62,   62,   62,   62,

       62,   63,   62,   62,   22,   62,   62,   62,   62,   62,
       62,   62,   62,   62,   62,   62,   62,   62,   62,   62,
       62,   62,   22,   47,   22,   48,   66,   67,  138,  166,
       22,   22,  144,  168,   22,   68,   69,  169,   49,   70,
       50,   51,  146,   22,   22,   22,   22,  179,  144,  180,
      180,  180,  180,  347,  147,  147,   62,   22,   23,   71,
       72,   23,   26,   27,   73,   29,   30,   31,   22,   22,
       33,   34,   35,   36,   74,   38,   39,   40,   40,   40,
       41,   42,   43,   44,   75,   75,   75,   75,   75,   76,
       75,   75,   22,   75,   75,   75,   75,   75,   75,   75,

       75,   75,   75,   75,   75,   75,   75,   75,   75,   75,
       22,   47,   22,   48,   78,   79,   80,   78,   81,  190,
      144,   82,   83,  346,   84,   22,   85,   86,   87,   88,
       89,   90,  188,  188,  188,  188,  345,   91,   92,   93,
      181,  181,  181,  181,  344,  158,  147,  144,  191,  144,
      221,  182,  147,  180,  180,  180,  180,  193,  193,  194,
      193,  144,  165,  138,  184,  138,  138,   94,  343,   95,
       78,   79,   80,   78,   81,  184,  144,   82,   83,  222,
       84,   22,   85,   86,   87,   88,   89,   90,  196,  196,
      197,  196,  144,   91,   92,   93,  201,  207,  207,  207,

      207,  206,  208,  208,  209,  208,  158,  144,  153,  153,
      153,  153,  342,  147,  147,  217,  217,  218,  217,  138,
      166,  255,  255,   94,  341,   95,   96,  258,  258,  258,
      203,  147,  261,  340,  204,  266,  266,  225,  144,  226,
      226,  226,  226,  290,  290,  147,   97,   97,   97,   97,
       97,   97,   97,   97,  140,   97,   97,   97,   97,   97,
       97,   97,   98,   97,   97,   99,   97,  100,   97,  101,
       97,   97,  103,  104,  105,  106,  107,  262,  244,  108,
      109,  169,  339,  141,  110,  111,  112,  113,  114,  115,
      175,  239,  239,  239,  239,  116,  117,  118,  225,  238,

      226,  226,  226,  226,  295,  190,  147,  245,  238,  232,
      336,  226,  226,  226,  226,  135,  135,  136,  135,  176,
      337,  221,  234,  255,  255,  119,  120,  121,  103,  104,
      105,  106,  107,  234,  191,  108,  109,  269,  269,  269,
      110,  111,  112,  113,  114,  115,  135,  135,  136,  135,
      222,  116,  117,  118,  135,  135,  136,  135,  242,  242,
      242,  242,  241,  153,  153,  153,  153,  251,  251,  251,
      251,  256,  338,  256,  266,  266,  257,  257,  257,  257,
      335,  119,  120,  121,   22,   23,   24,   25,   23,   26,
      122,  123,   29,   30,  124,   22,  125,   33,  126,  127,

      128,  129,   38,  130,  131,  131,  131,   22,   42,   43,
       44,  132,  132,  132,  132,  132,  133,  132,  132,   22,
      132,  132,  132,  132,  132,  132,  132,  132,  132,  132,
      132,  132,  132,  132,  132,  132,  132,  134,   47,   22,
       48,  150,  151,  290,  290,  152,  159,  334,  149,  149,
      149,  149,  333,  244,  153,  153,  153,  153,  332,  161,
      147,  171,  171,  172,  171,  331,  330,  155,  329,  328,
      161,  171,  171,  172,  171,  293,  293,  293,  156,  327,
      157,  159,  245,  149,  149,  149,  149,  326,  153,  153,
      153,  153,  325,  160,  161,  324,  323,  162,  163,  249,

      322,  321,  160,  320,  319,  161,  171,  171,  172,  171,
      249,  162,  182,  318,  180,  180,  180,  180,  317,  254,
      254,  254,  254,  316,  183,  184,  315,  314,  185,  186,
      249,  313,  312,  183,  311,  310,  184,  263,  263,  263,
      263,  249,  185,  150,  151,  307,  306,  152,  181,  181,
      181,  181,  181,  181,  181,  181,  227,  227,  227,  227,
      305,  272,  147,  264,  188,  188,  188,  188,  182,  229,
      180,  180,  180,  180,  264,  193,  193,  194,  193,  147,
      230,  184,  231,  232,  301,  226,  226,  226,  226,  258,
      258,  258,  184,  300,  244,  233,  234,  241,  238,  235,

      236,  269,  269,  269,  233,  288,  287,  234,  265,  265,
      265,  265,  221,  235,  282,  267,  281,  267,  280,  264,
      268,  268,  268,  268,  193,  193,  194,  193,  279,  206,
      264,  193,  193,  194,  193,  196,  196,  197,  196,  196,
      196,  197,  196,  196,  196,  197,  196,  273,  273,  273,
      273,  272,  207,  207,  207,  207,  206,  208,  208,  209,
      208,  208,  208,  209,  208,  208,  208,  209,  208,  217,
      217,  218,  217,  217,  217,  218,  217,  217,  217,  218,
      217,  283,  283,  283,  283,  241,  227,  227,  227,  227,
      232,  277,  226,  226,  226,  226,  227,  227,  227,  227,

      293,  293,  293,  234,  276,  272,  190,  284,  286,  286,
      286,  286,  175,  147,  234,  147,  253,  252,  284,  289,
      289,  289,  289,  244,  247,  144,  291,  140,  291,  241,
      284,  292,  292,  292,  292,  239,  239,  239,  239,  238,
      221,  284,  223,  238,  242,  242,  242,  242,  241,  297,
      297,  297,  297,  298,  147,  298,  147,  147,  299,  299,
      299,  299,  251,  251,  251,  251,  254,  254,  254,  254,
      257,  257,  257,  257,  144,  138,  219,  249,  257,  257,
      257,  257,  263,  263,  263,  263,  216,  302,  249,  302,
      215,  214,  303,  303,  303,  303,  265,  265,  265,  265,

      268,  268,  268,  268,  213,  147,  147,  264,  268,  268,
      268,  268,  273,  273,  273,  273,  272,  147,  264,  283,
      283,  283,  283,  241,  308,  144,  308,  138,  210,  309,
      309,  309,  309,  286,  286,  286,  286,  289,  289,  289,
      289,  292,  292,  292,  292,  206,  200,  198,  284,  292,
      292,  292,  292,  297,  297,  297,  297,  195,  190,  284,
      299,  299,  299,  299,  299,  299,  299,  299,  303,  303,
      303,  303,  303,  303,  303,  303,  309,  309,  309,  309,
      309,  309,  309,  309,   22,   22,   22,   22,   22,   22,
       22,   22,   77,   77,   77,   77,   77,   77,   77,   77,

      102,  102,  102,  102,  102,  102,  102,  102,  139,  175,
      139,  139,  139,  139,  139,  139,  142,  142,  145,  177,
      145,  145,  145,  145,  145,  145,  154,  154,  154,  173,
      154,  154,  164,  164,  167,  167,  167,  167,  174,  147,
      174,  174,  174,  174,  174,  174,  178,  147,  178,  178,
      178,  178,  178,  178,  187,  147,  187,  187,  189,  140,
      189,  189,  189,  189,  189,  189,  192,  144,  192,  192,
      192,  192,  192,  192,  199,  199,  202,  202,  143,  202,
      202,  205,  205,  205,  205,  211,  211,  212,  138,  212,
      212,  220,  137,  220,  220,  220,  220,  220,  220,  224,

      351,  224,  224,  224,  224,  224,  224,  228,  228,  228,
      351,  228,  228,  237,  351,  237,  237,  240,  240,  240,
      240,  243,  351,  243,  243,  243,  243,  243,  243,  246,
      246,  248,  351,  248,  248,  248,  248,  248,  248,  250,
      351,  250,  250,  259,  351,  351,  259,  260,  351,  260,
      260,  270,  351,  351,  270,  271,  271,  271,  271,  274,
      274,  275,  351,  275,  275,  278,  351,  278,  278,  285,
      351,  285,  285,  294,  351,  351,  294,  296,  351,  296,
      296,  304,  351,  304,  304,   21,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,

      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351
    } ;

static yyconst flex_int16_t yy_chk[1344] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    3,    3,    3,
        3,    3,    7,    8,    3,    3,    7,    8,   27,    3,
        3,    3,    3,    3,    3,   24,   24,   24,   24,   30,
        3,    3,    3,   34,   33,   34,   34,   34,   34,   42,
       42,   34,   38,   43,   43,   30,   33,   27,   36,   38,

       36,   36,   36,   36,   44,   44,   36,   45,   83,   48,
        3,   45,    3,    4,    4,    4,    4,    4,   91,   91,
        4,    4,   47,  346,   83,    4,    4,    4,    4,    4,
        4,   52,   52,   52,   52,   54,    4,    4,    4,   40,
       48,   40,   40,   40,   40,   46,   46,   46,   46,   46,
      345,   47,   40,   46,   57,   85,   57,   57,   57,   57,
       92,   92,   57,   40,   54,  344,    4,   85,    4,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,

        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    9,    9,   93,   93,
        9,    9,  109,  167,    9,    9,    9,  167,    9,    9,
        9,    9,  110,    9,    9,    9,    9,   58,  109,   58,
       58,   58,   58,  343,  110,   58,    9,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,

       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   13,   13,   13,   13,   13,   64,
       94,   13,   13,  342,   13,   13,   13,   13,   13,   13,
       13,   13,   63,   63,   63,   63,  341,   13,   13,   13,
       59,   59,   59,   59,  340,   90,   59,  119,   64,   94,
      122,   61,   90,   61,   61,   61,   61,   66,   66,   66,
       66,   95,  116,  116,   61,  117,  117,   13,  339,   13,
       14,   14,   14,   14,   14,   61,  119,   14,   14,  122,
       14,   14,   14,   14,   14,   14,   14,   14,   71,   71,
       71,   71,   95,   14,   14,   14,   74,   76,   76,   76,

       76,   76,   79,   79,   79,   79,  115,  121,   74,   74,
       74,   74,  338,  115,   74,  104,  104,  104,  104,  118,
      118,  160,  160,   14,  337,   14,   15,  162,  162,  162,
       74,  166,  166,  336,   74,  183,  183,  126,  121,  126,
      126,  126,  126,  233,  233,  126,   15,   15,   15,   15,
       15,   15,   15,   15,  139,   15,   15,   15,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   17,   17,   17,   17,   17,  169,  150,   17,
       17,  169,  335,  139,   17,   17,   17,   17,   17,   17,
      174,  133,  133,  133,  133,   17,   17,   17,  128,  133,

      128,  128,  128,  128,  238,  189,  128,  150,  238,  131,
      332,  131,  131,  131,  131,  135,  135,  135,  135,  174,
      332,  220,  131,  255,  255,   17,   17,   17,   18,   18,
       18,   18,   18,  131,  189,   18,   18,  185,  185,  185,
       18,   18,   18,   18,   18,   18,  136,  136,  136,  136,
      220,   18,   18,   18,  137,  137,  137,  137,  143,  143,
      143,  143,  143,  148,  148,  148,  148,  155,  155,  155,
      155,  161,  334,  161,  266,  266,  161,  161,  161,  161,
      331,   18,   18,   18,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,

       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       19,   37,   37,  290,  290,   37,  149,  329,  149,  149,
      149,  149,  328,  243,   37,   37,   37,   37,  327,  149,
       37,  171,  171,  171,  171,  326,  325,   37,  324,  323,
      149,  172,  172,  172,  172,  235,  235,  235,   37,  321,
       37,   39,  243,   39,   39,   39,   39,  320,  153,  153,
      153,  153,  319,   39,   39,  318,  317,   39,   39,  153,

      316,  315,   39,  313,  312,   39,  173,  173,  173,  173,
      153,   39,   60,  311,   60,   60,   60,   60,  310,  159,
      159,  159,  159,  307,   60,   60,  306,  305,   60,   60,
      159,  301,  300,   60,  288,  287,   60,  177,  177,  177,
      177,  159,   60,  129,  129,  282,  277,  129,  179,  179,
      179,  179,  181,  181,  181,  181,  129,  129,  129,  129,
      276,  271,  129,  181,  188,  188,  188,  188,  180,  129,
      180,  180,  180,  180,  181,  193,  193,  193,  193,  261,
      129,  180,  129,  130,  253,  130,  130,  130,  130,  258,
      258,  258,  180,  252,  248,  130,  130,  240,  237,  130,

      130,  269,  269,  269,  130,  231,  230,  130,  182,  182,
      182,  182,  224,  130,  216,  184,  215,  184,  214,  182,
      184,  184,  184,  184,  194,  194,  194,  194,  213,  205,
      182,  195,  195,  195,  195,  196,  196,  196,  196,  197,
      197,  197,  197,  198,  198,  198,  198,  200,  200,  200,
      200,  200,  207,  207,  207,  207,  207,  208,  208,  208,
      208,  209,  209,  209,  209,  210,  210,  210,  210,  217,
      217,  217,  217,  218,  218,  218,  218,  219,  219,  219,
      219,  223,  223,  223,  223,  223,  225,  225,  225,  225,
      226,  204,  226,  226,  226,  226,  227,  227,  227,  227,

      293,  293,  293,  226,  203,  199,  192,  227,  229,  229,
      229,  229,  178,  165,  226,  158,  157,  156,  227,  232,
      232,  232,  232,  152,  151,  146,  234,  145,  234,  142,
      232,  234,  234,  234,  234,  239,  239,  239,  239,  132,
      124,  232,  123,  239,  242,  242,  242,  242,  242,  247,
      247,  247,  247,  249,  114,  249,  113,  111,  249,  249,
      249,  249,  251,  251,  251,  251,  254,  254,  254,  254,
      256,  256,  256,  256,  108,  107,  105,  254,  257,  257,
      257,  257,  263,  263,  263,  263,  101,  264,  254,  264,
      100,   99,  264,  264,  264,  264,  265,  265,  265,  265,

      267,  267,  267,  267,   98,   89,   88,  265,  268,  268,
      268,  268,  273,  273,  273,  273,  273,   86,  265,  283,
      283,  283,  283,  283,  284,   82,  284,   81,   80,  284,
      284,  284,  284,  286,  286,  286,  286,  289,  289,  289,
      289,  291,  291,  291,  291,   75,   73,   72,  289,  292,
      292,  292,  292,  297,  297,  297,  297,   67,   65,  289,
      298,  298,  298,  298,  299,  299,  299,  299,  302,  302,
      302,  302,  303,  303,  303,  303,  308,  308,  308,  308,
      309,  309,  309,  309,  352,  352,  352,  352,  352,  352,
      352,  352,  353,  353,  353,  353,  353,  353,  353,  353,

      354,  354,  354,  354,  354,  354,  354,  354,  355,   56,
      355,  355,  355,  355,  355,  355,  356,  356,  357,   55,
      357,  357,  357,  357,  357,  357,  358,  358,  358,   53,
      358,  358,  359,  359,  360,  360,  360,  360,  361,   51,
      361,  361,  361,  361,  361,  361,  362,   50,  362,  362,
      362,  362,  362,  362,  363,   49,  363,  363,  364,   31,
      364,  364,  364,  364,  364,  364,  365,   29,  365,  365,
      365,  365,  365,  365,  366,  366,  367,  367,   28,  367,
      367,  368,  368,  368,  368,  369,  369,  370,   26,  370,
      370,  371,   25,  371,  371,  371,  371,  371,  371,  372,

       21,  372,  372,  372,  372,  372,  372,  373,  373,  373,
        0,  373,  373,  374,    0,  374,  374,  375,  375,  375,
      375,  376,    0,  376,  376,  376,  376,  376,  376,  377,
      377,  378,    0,  378,  378,  378,  378,  378,  378,  379,
        0,  379,  379,  380,    0,    0,  380,  381,    0,  381,
      381,  382,    0,    0,  382,  383,  383,  383,  383,  384,
      384,  385,    0,  385,  385,  386,    0,  386,  386,  387,
        0,  387,  387,  388,    0,    0,  388,  389,    0,  389,
      389,  390,    0,  390,  390,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,

      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351,  351,  351,  351,  351,  351,  351,  351,
      351,  351,  351
    } ;

/* Table of booleans, true if rule could match eol. */
static yyconst flex_int32_t yy_rule_can_match_eol[73] =
    {   0,
1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,     };

static yyconst flex_int16_t yy_rule_linenum[72] =
    {   0,
      199,  205,  211,  216,  221,  224,  229,  234,  239,  243,
      248,  259,  264,  269,  273,  277,  280,  284,  290,  300,
      308,  312,  316,  321,  327,  337,  342,  346,  351,  356,
      358,  363,  368,  373,  389,  394,  399,  423,  438,  446,
      453,  467,  483,  491,  497,  501,  505,  509,  513,  517,
      525,  530,  533,  536,  539,  542,  546,  551,  556,  563,
      568,  573,  577,  581,  588,  593,  597,  601,  610,  612,
      617
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "macro.l"

#line 23 "macro.l"

/*
 * $Id: /mirror/trunk/compilers/pirc/macro/macrolexer.c 33586 2008-12-06T13:00:56.132030Z moritz  $
 * Copyright (C) 2007-2008, The Perl Foundation.
 */

/*
 * This pre-processor handles the macro layer of the
 * PIR language. The following constructs are filtered
 * out and handled:
 *
 * .include
 * .macro
 * .macro_local
 * .macro_label
 * .macro_const
 *
 */


#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include "macro.h"
#include "lexer.h"

#include "parrot/parrot.h"

#define  YY_NO_UNISTD_H

#include "macroparser.h"

#define YY_EXTRA_TYPE  struct lexer_state *


extern macro_def *find_macro(constant_table *table, char *name);
extern char *concat(char *str1, char *str2);

int  macroget_column(yyscan_t yyscanner);
void macroset_column(int col, yyscan_t yyscanner);

char *dupstrn(char const * const source, size_t num_chars);
char *dupstr(char const * const source);

/*

=head1 FUNCTIONS

=over 4

=item C<char *
dupstrn(char const * const source, size_t num_chars)>

See dupstr, except that this version takes the number of characters to be
copied. Easy for copying a string except the quotes.

=cut

*/
PARROT_MALLOC
PARROT_WARN_UNUSED_RESULT
PARROT_CANNOT_RETURN_NULL
char *
dupstrn(char const * const source, size_t num_chars) {
    char *newstring = (char *)mem_sys_allocate_zeroed((num_chars + 1) * sizeof (char));
    assert(newstring);
    /* only copy num_chars characters */
    strncpy(newstring, source, num_chars);
    return newstring;
}


/*

=item C<char *
dupstr(char const * const source)>

The C89 standard does not define a strdup() in the C library,
so define our own strdup. Function names beginning with "str"
are reserved (I think), so make it dupstr, as that is what it
does: duplicate a string.

=cut

*/
PARROT_WARN_UNUSED_RESULT
PARROT_CANNOT_RETURN_NULL
char *
dupstr(char const * const source) {
    return dupstrn(source, strlen(source));
}


/*

=back

=cut

*/

/*
Use these on a big file with macros, and check out whether there are unused rules.
*/
int ctr[YY_NUM_RULES];
int num_rules = YY_NUM_RULES;


#define YY_USER_ACTION   { ++ctr[yy_act]; }



static int is_pir_directive(char const * const name);










#line 1116 "macrolexer.c"

#define INITIAL 0
#define MACRODEF 1
#define CONSTDEF 2
#define INCLUDE 3
#define MACROPAR 4
#define MACROBODY 5
#define EXPAND 6
#define MACROLOCAL 7
#define BRACEDARGS 8
#define EXPARGS 9

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals (yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r

int macrolex_init (yyscan_t* scanner);

int macrolex_init_extra (YY_EXTRA_TYPE user_defined,yyscan_t* scanner);

/* %endif */

/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int macrolex_destroy (yyscan_t yyscanner );

int macroget_debug (yyscan_t yyscanner );

void macroset_debug (int debug_flag ,yyscan_t yyscanner );

YY_EXTRA_TYPE macroget_extra (yyscan_t yyscanner );

void macroset_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );

FILE *macroget_in (yyscan_t yyscanner );

void macroset_in  (FILE * in_str ,yyscan_t yyscanner );

FILE *macroget_out (yyscan_t yyscanner );

void macroset_out  (FILE * out_str ,yyscan_t yyscanner );

int macroget_leng (yyscan_t yyscanner );

char *macroget_text (yyscan_t yyscanner );

int macroget_lineno (yyscan_t yyscanner );

void macroset_lineno (int line_number ,yyscan_t yyscanner );

/* %if-bison-bridge */

YYSTYPE * macroget_lval (yyscan_t yyscanner );

void macroset_lval (YYSTYPE * yylval_param ,yyscan_t yyscanner );

/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int macrowrap (yyscan_t yyscanner );
#else
extern int macrowrap (yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */

/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

    static void yy_push_state (int new_state ,yyscan_t yyscanner);

    static void yy_pop_state (yyscan_t yyscanner );

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO fwrite( yytext, yyleng, 1, yyout )
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
    if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
        { \
        int c = '*'; \
        int n; \
        for ( n = 0; n < max_size && \
                 (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
            buf[n] = (char) c; \
        if ( c == '\n' ) \
            buf[n++] = (char) c; \
        if ( c == EOF && ferror( yyin ) ) \
            YY_FATAL_ERROR( "input in flex scanner failed" ); \
        result = n; \
        } \
    else \
        { \
        errno=0; \
        while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
            { \
            if( errno != EINTR) \
                { \
                YY_FATAL_ERROR( "input in flex scanner failed" ); \
                break; \
                } \
            errno=0; \
            clearerr(yyin); \
            } \
        }\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */

/* %ok-for-header */

/* %not-for-header */

/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int macrolex \
               (YYSTYPE * yylval_param ,yyscan_t yyscanner);

#define YY_DECL int macrolex \
               (YYSTYPE * yylval_param , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
    YY_USER_ACTION

/* %not-for-header */

/** The main scanner function which does all the work.
 */
YY_DECL
{
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [7.0] user's declarations go here */
#line 195 "macro.l"




#line 1430 "macrolexer.c"

    yylval = yylval_param;

    if ( !yyg->yy_init )
        {
        yyg->yy_init = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if ( ! yyg->yy_start )
            yyg->yy_start = 1;  /* first start state */

        if ( ! yyin )
/* %if-c-only */
            yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! yyout )
/* %if-c-only */
            yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

        if ( ! YY_CURRENT_BUFFER ) {
            macroensure_buffer_stack (yyscanner);
            YY_CURRENT_BUFFER_LVALUE =
                macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
        }

        macro_load_buffer_state(yyscanner );
        }

    while ( 1 )     /* loops until end-of-file is reached */
        {
/* %% [8.0] yymore()-related code goes here */
        yy_cp = yyg->yy_c_buf_p;

        /* Support of yytext. */
        *yy_cp = yyg->yy_hold_char;

        /* yy_bp points to the position in yy_ch_buf of the start of
         * the current run.
         */
        yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
        yy_current_state = yyg->yy_start;
yy_match:
        do
            {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if ( yy_accept[yy_current_state] )
                {
                yyg->yy_last_accepting_state = yy_current_state;
                yyg->yy_last_accepting_cpos = yy_cp;
                }
            while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
                {
                yy_current_state = (int) yy_def[yy_current_state];
                if ( yy_current_state >= 352 )
                    yy_c = yy_meta[(unsigned int) yy_c];
                }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
            ++yy_cp;
            }
        while ( yy_current_state != 351 );
        yy_cp = yyg->yy_last_accepting_cpos;
        yy_current_state = yyg->yy_last_accepting_state;

yy_find_action:
/* %% [10.0] code to find the action number goes here */
        yy_act = yy_accept[yy_current_state];

        YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

        if ( yy_act != YY_END_OF_BUFFER && yy_rule_can_match_eol[yy_act] )
            {
            int yyl;
            for ( yyl = 0; yyl < yyleng; ++yyl )
                if ( yytext[yyl] == '\n' )

    do{ yylineno++;
        yycolumn=0;
    }while(0)
;
            }

do_action:  /* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
        if ( yy_flex_debug )
            {
            if ( yy_act == 0 )
                fprintf( stderr, "--scanner backing up\n" );
            else if ( yy_act < 72 )
                fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
                         (long)yy_rule_linenum[yy_act], yytext );
            else if ( yy_act == 72 )
                fprintf( stderr, "--accepting default rule (\"%s\")\n",
                         yytext );
            else if ( yy_act == 73 )
                fprintf( stderr, "--(end of buffer or a NUL)\n" );
            else
                fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
            }

        switch ( yy_act )
    { /* beginning of action switch */
/* %% [13.0] actions go here */
            case 0: /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = yyg->yy_hold_char;
            yy_cp = yyg->yy_last_accepting_cpos;
            yy_current_state = yyg->yy_last_accepting_state;
            goto yy_find_action;

case 1:
/* rule 1 can match eol */
YY_RULE_SETUP
#line 199 "macro.l"
{ /**/
                      yy_pop_state(yyscanner);
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 2:
/* rule 2 can match eol */
YY_RULE_SETUP
#line 205 "macro.l"
{ /* after macro header start the macro body */
                      yy_push_state(MACROBODY, yyscanner);
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 3:
/* rule 3 can match eol */
YY_RULE_SETUP
#line 211 "macro.l"
{ /* return newlines in macro body as any tokens */
                       yylval->sval = dupstr("\n");
                       return TK_ANY;
                     }
    YY_BREAK
case 4:
/* rule 4 can match eol */
YY_RULE_SETUP
#line 216 "macro.l"
{ yy_pop_state(yyscanner);
                      yylval->sval = "\n";
                      return TK_NL;
                    }
    YY_BREAK
case 5:
/* rule 5 can match eol */
YY_RULE_SETUP
#line 221 "macro.l"
{ yylval->sval = "\n"; return TK_ANY; }
    YY_BREAK
case 6:
YY_RULE_SETUP
#line 224 "macro.l"
{ /* */
                      yy_push_state(INCLUDE, yyscanner);
                      return TK_INCLUDE;
                    }
    YY_BREAK
case 7:
YY_RULE_SETUP
#line 229 "macro.l"
{ /* */
                      yy_push_state(CONSTDEF, yyscanner);
                      return TK_MACRO_CONST;
                    }
    YY_BREAK
case 8:
YY_RULE_SETUP
#line 234 "macro.l"
{ /* */
                      yy_push_state(MACROPAR, yyscanner);
                      return TK_MACRO;
                    }
    YY_BREAK
case 9:
YY_RULE_SETUP
#line 239 "macro.l"
{ /* */
                            return TK_MACRO_LABEL;
                          }
    YY_BREAK
case 10:
YY_RULE_SETUP
#line 243 "macro.l"
{ /* unique label declaration using macro parameter */
                            yylval->sval = dupstr(yytext);
                            return TK_LABEL_ID;
                          }
    YY_BREAK
case 11:
YY_RULE_SETUP
#line 248 "macro.l"
{ /* unique label declaration, add special marker. */
                          char * temp = (char *)mem_sys_allocate((yyleng + 1 + 1) * sizeof (char));
                          /* stick a special marker "@" so we can recognize this as a label that
                           * must be munged.
                           */
                          strncpy(temp, yytext, yyleng - 1);
                          strcpy(temp + yyleng - 1, "@:");
                          yylval->sval = temp;
                          return TK_LABEL_ID;
                        }
    YY_BREAK
case 12:
YY_RULE_SETUP
#line 259 "macro.l"
{ /* referring to a declared label or local id. */
                            yylval->sval = dupstr(yytext );
                            return TK_ANY;
                          }
    YY_BREAK
case 13:
YY_RULE_SETUP
#line 264 "macro.l"
{ /* unique local declaration */
                            yy_push_state(MACROLOCAL, yyscanner);
                            return TK_MACRO_LOCAL;
                          }
    YY_BREAK
case 14:
YY_RULE_SETUP
#line 269 "macro.l"
{ yylval->sval = dupstr("int");
                            return TK_INT;
                          }
    YY_BREAK
case 15:
YY_RULE_SETUP
#line 273 "macro.l"
{ yylval->sval = dupstr("num");
                            return TK_NUM;
                          }
    YY_BREAK
case 16:
YY_RULE_SETUP
#line 277 "macro.l"
{ yylval->sval = dupstr("pmc");
                            return TK_PMC;
                          }
    YY_BREAK
case 17:
YY_RULE_SETUP
#line 280 "macro.l"
{ yylval->sval = dupstr("string");
                            return TK_STRING;
                          }
    YY_BREAK
case 18:
YY_RULE_SETUP
#line 284 "macro.l"
{ /* unique local id declaration */
                            yylval->sval = dupstr(yytext);
                            yy_pop_state(yyscanner);
                            return TK_LOCAL_ID;
                          }
    YY_BREAK
case 19:
YY_RULE_SETUP
#line 290 "macro.l"
{ /* unique local id declaration -- non-parameter */
                         char * temp = (char *)mem_sys_allocate((yyleng + 1 + 1) * sizeof (char));
                         strcpy(temp, yytext);
                         /* add special marker that this token needs munging later */
                         temp[yyleng] = '@';
                         yylval->sval = temp;
                         yy_pop_state(yyscanner);
                         return TK_LOCAL_ID;
                       }
    YY_BREAK
case 20:
YY_RULE_SETUP
#line 300 "macro.l"
{ /* when reading ".endm" while scanning macro body, go back to previous state */

                     yy_pop_state(yyscanner);
                     yy_pop_state(yyscanner);

                     return TK_ENDM;
                   }
    YY_BREAK
case 21:
YY_RULE_SETUP
#line 308 "macro.l"
{ /* recognize "(" when scanning macro parameter list */
                     return '(';
                   }
    YY_BREAK
case 22:
YY_RULE_SETUP
#line 312 "macro.l"
{ /* recognize "," when scanning macro parameter list */
                     return ',';
                   }
    YY_BREAK
case 23:
YY_RULE_SETUP
#line 316 "macro.l"
{ /* recognize ")" when scanning a macro parameter list */
                     return ')';
                   }
    YY_BREAK
case 24:
YY_RULE_SETUP
#line 321 "macro.l"
{ /* recognize "(" when expanding a macro */
                     yy_push_state(EXPARGS, yyscanner);
                     return '(';
                   }
    YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
#line 327 "macro.l"
{ /* anything except a ")" in this state means a constant expansion, which
                      * has no arguments. This works, because if we did see a '(', the
                      * state EXPARGS is activated.
                      */
                     yylval->sval = dupstr(yytext);
                     yy_pop_state(yyscanner);
                     return TK_ANY;
                   }
    YY_BREAK
case 26:
YY_RULE_SETUP
#line 337 "macro.l"
{ /* */
                    yylval->sval = dupstr(yytext);
                    return TK_ANY;
                  }
    YY_BREAK
case 27:
YY_RULE_SETUP
#line 342 "macro.l"
{ /* */
                     return ',';
                   }
    YY_BREAK
case 28:
YY_RULE_SETUP
#line 346 "macro.l"
{ /* */
                     yy_pop_state(yyscanner);
                     return ')';
                   }
    YY_BREAK
case 29:
YY_RULE_SETUP
#line 351 "macro.l"
{ /* braced argument */
                     yy_push_state(BRACEDARGS, yyscanner);
                     return '{';
                   }
    YY_BREAK
case 30:
YY_RULE_SETUP
#line 356 "macro.l"
{ yylval->sval = dupstr(" "); return TK_ANY; }
    YY_BREAK
case 31:
/* rule 31 can match eol */
YY_RULE_SETUP
#line 358 "macro.l"
{
                     yylval->sval = dupstr(yytext);
                     return TK_ANY;
                   }
    YY_BREAK
case 32:
YY_RULE_SETUP
#line 363 "macro.l"
{
                     yy_pop_state(yyscanner);
                     return '}';
                   }
    YY_BREAK
case 33:
YY_RULE_SETUP
#line 368 "macro.l"
{ /* variable argument of macro expansion */
                     yylval->sval = dupstr(yytext);
                     return TK_IDENT;
                   }
    YY_BREAK
case 34:
YY_RULE_SETUP
#line 373 "macro.l"
{ /* .foo(.bar) */
                      lexer_state *lexer = macroget_extra(yyscanner);
                      macro_def *macro = find_macro(lexer->globaldefinitions, dupstr(yytext + 1));

                      if (macro != NULL) {
                         yylval->sval = macro->body;
                         return TK_IDENT;
                      }
                      else {
                         fprintf(stderr, "Error: cannot find expansion for %s\n", yytext + 1);
                         lexer->errors++;
                      }
                    }
    YY_BREAK
case 35:
YY_RULE_SETUP
#line 389 "macro.l"
{ /* register can be an argument for macro expansion. */
                     yylval->sval = dupstr(yytext);
                     return TK_IDENT;
                   }
    YY_BREAK
case 36:
YY_RULE_SETUP
#line 394 "macro.l"
{ /* constant or macro parameter ID */
                               yylval->sval = dupstr(yytext);
                               return TK_IDENT;
                             }
    YY_BREAK
case 37:
YY_RULE_SETUP
#line 399 "macro.l"
{ /* directive, constant, macro or method-call */
                      if (is_pir_directive(yytext)) {
                          yylval->sval = dupstr(yytext);
                          return TK_ANY;
                      }
                      else { /* not a directive */
                          lexer_state *lexer = macroget_extra(yyscanner);
                          macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 1);
                          if (macro != NULL) { /* it's a macro! */
                              /* only go into EXPAND state if it's a macro/constant */
                              yy_push_state(EXPAND, yyscanner);
                              yylval->mval = macro; /* pass the pointer to the macro_def */
                              return TK_DOT_IDENT;
                          }
                          else { /* it's not a macro, just a method-call, but if it was supposed
                                    to be a macro expansion, we will only see the error in the pir
                                    compiler.
                                  */
                              yylval->sval = dupstr(yytext);
                              return TK_ANY;
                          }
                      }
                   }
    YY_BREAK
case 38:
YY_RULE_SETUP
#line 423 "macro.l"
{ /* */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, /* skip "$" and ":" */
                                                   dupstrn(yytext + 1, yyleng - 2));

                     if (macro != NULL) { /* found the parameter */
                        yylval->sval = macro->body; /* expand the macro-parameter */
                        return TK_LABEL_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for '%s'\n", yytext);
                        lexer->errors++;
                     }
                   }
    YY_BREAK
case 39:
YY_RULE_SETUP
#line 438 "macro.l"
{ /* this label was declared unique by ".macro_label foo:" but was munged so we
                        can recognize it now, and return a signal to the parser it needs to
                        munge it into a unique ID.
                      */
                     yylval->sval = dupstrn(yytext, yyleng - 2);
                     return TK_UNIQUE_LABEL;
                   }
    YY_BREAK
case 40:
YY_RULE_SETUP
#line 446 "macro.l"
{ /* this local was declared unique by ".macro_local", but was munged so we can
                        recognize it now.
                     */
                     yylval->sval = dupstrn(yytext, yyleng - 1);
                     return TK_UNIQUE_LOCAL;
                   }
    YY_BREAK
case 41:
YY_RULE_SETUP
#line 453 "macro.l"
{   /* expanding a declared variable (local or label) */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 2);

                     if (macro != NULL) { /**/
                        yylval->sval = macro->body;
                        return TK_VAR_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for symbol '%s'\n", yytext);
                        lexer->errors++;
                     }
                 }
    YY_BREAK
case 42:
YY_RULE_SETUP
#line 467 "macro.l"
{    /* */
                     lexer_state *lexer = macroget_extra(yyscanner);
                     macro_def *macro = find_macro(lexer->globaldefinitions, yytext + 1);
                     if (macro != NULL) { /*  */
                        yylval->sval = macro->body;
                        return TK_VAR_EXPANSION;
                     }
                     else {
                        fprintf(stderr, "Error: cannot find expansion for parameter '%s'\n",
                                yytext + 1);
                        lexer->errors++;
                     }


                   }
    YY_BREAK
case 43:
YY_RULE_SETUP
#line 483 "macro.l"
{/* treat ".'foo'" or ".$P0" as 1 token to prevent the tokens are separated
                        * by a space; they clearly belong to each other if they were written
                        * together (without a space). This is a method-call, life foo .'bar'().
                        */
                       yylval->sval = dupstr(yytext);
                       return TK_ANY;
                      }
    YY_BREAK
case 44:
YY_RULE_SETUP
#line 491 "macro.l"
{ /* in all other cases (than the above), treat a comma just as any token. */
                        yylval->sval = dupstr(yytext);
                        return TK_ANY;
                      }
    YY_BREAK
case 45:
YY_RULE_SETUP
#line 497 "macro.l"
{ /**/
                       yylval->sval = dupstr(yytext);
                       return TK_IDENT;
                     }
    YY_BREAK
case 46:
YY_RULE_SETUP
#line 501 "macro.l"
{ yylval->sval = dupstr(yytext);
                       return TK_STRINGC;
                     }
    YY_BREAK
case 47:
YY_RULE_SETUP
#line 505 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 48:
YY_RULE_SETUP
#line 509 "macro.l"
{ yylval->sval = dupstr(yytext);
                                return TK_INTC;
                              }
    YY_BREAK
case 49:
YY_RULE_SETUP
#line 513 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_NUMC;
                          }
    YY_BREAK
case 50:
YY_RULE_SETUP
#line 517 "macro.l"
{ /* a quoted string is needed for an .include or the value of a
                             * macro constant. After this token, leave the current state.
                             */
                             yylval->sval = dupstr(yytext);
                             yy_pop_state(yyscanner);
                             return TK_STRINGC;
                          }
    YY_BREAK
case 51:
YY_RULE_SETUP
#line 525 "macro.l"
{ /* quoted strings argument for macro expansion */
                            yylval->sval = dupstr(yytext);
                            return TK_STRINGC;
                          }
    YY_BREAK
case 52:
YY_RULE_SETUP
#line 530 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 53:
YY_RULE_SETUP
#line 533 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 54:
YY_RULE_SETUP
#line 536 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 55:
YY_RULE_SETUP
#line 539 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_INTC;
                          }
    YY_BREAK
case 56:
YY_RULE_SETUP
#line 542 "macro.l"
{ yylval->sval = dupstr(yytext);
                            return TK_NUMC;
                          }
    YY_BREAK
case 57:
YY_RULE_SETUP
#line 546 "macro.l"
{ /* register */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 58:
YY_RULE_SETUP
#line 551 "macro.l"
{ /* quoted string */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 59:
YY_RULE_SETUP
#line 556 "macro.l"
{ /* identifier */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 60:
YY_RULE_SETUP
#line 563 "macro.l"
{ /* flag */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 61:
YY_RULE_SETUP
#line 568 "macro.l"
{ /* expansions in a macro body; ignore for now. */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 62:
YY_RULE_SETUP
#line 573 "macro.l"
{ yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 63:
YY_RULE_SETUP
#line 577 "macro.l"
{ yylval->sval = dupstr(yytext);
                                     return TK_ANY;
                                   }
    YY_BREAK
case 64:
YY_RULE_SETUP
#line 581 "macro.l"
{ yylval->sval = dupstr(yytext);
                                         return TK_ANY;
                                       }
    YY_BREAK
case 65:
YY_RULE_SETUP
#line 588 "macro.l"
{ /* normal label */
                                   yylval->sval = dupstr(yytext);
                                   return TK_ANY;
                                 }
    YY_BREAK
case 66:
YY_RULE_SETUP
#line 593 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case 67:
YY_RULE_SETUP
#line 597 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case 68:
YY_RULE_SETUP
#line 601 "macro.l"
{ yylval->sval = dupstr(yytext);
                                                          return TK_ANY;
                                                        }
    YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(MACRODEF):
case YY_STATE_EOF(CONSTDEF):
case YY_STATE_EOF(INCLUDE):
case YY_STATE_EOF(MACROPAR):
case YY_STATE_EOF(MACROBODY):
case YY_STATE_EOF(EXPAND):
case YY_STATE_EOF(MACROLOCAL):
case YY_STATE_EOF(BRACEDARGS):
case YY_STATE_EOF(EXPARGS):
#line 605 "macro.l"
{ /* when end of file or end of string buffer, stop scanning. */
                      yyterminate();
                    }
    YY_BREAK
case 69:
YY_RULE_SETUP
#line 610 "macro.l"
{ /* skip whitespace */ }
    YY_BREAK
case 70:
/* rule 70 can match eol */
YY_RULE_SETUP
#line 612 "macro.l"
{ /* in all other states, return newline as the newline token */
                      yylval->sval = dupstr("\n");
                      return TK_NL;
                    }
    YY_BREAK
case 71:
YY_RULE_SETUP
#line 617 "macro.l"
{ /* just return any single character token we didn't match before. */
                         yylval->sval = dupstr(yytext);
                         return TK_ANY;
                       }
    YY_BREAK
case 72:
YY_RULE_SETUP
#line 622 "macro.l"
ECHO;
    YY_BREAK
#line 2207 "macrolexer.c"

    case YY_END_OF_BUFFER:
        {
        /* Amount of text matched not including the EOB char. */
        int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

        /* Undo the effects of YY_DO_BEFORE_ACTION. */
        *yy_cp = yyg->yy_hold_char;
        YY_RESTORE_YY_MORE_OFFSET

        if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
            {
            /* We're scanning a new file or input source.  It's
             * possible that this happened because the user
             * just pointed yyin at a new source and called
             * macrolex().  If so, then we have to assure
             * consistency between YY_CURRENT_BUFFER and our
             * globals.  Here is the right place to do so, because
             * this is the first action (other than possibly a
             * back-up) that will match for the new input source.
             */
            yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
            YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

        /* Note that here we test for yy_c_buf_p "<=" to the position
         * of the first EOB in the buffer, since yy_c_buf_p will
         * already have been incremented past the NUL character
         * (since all states make transitions on EOB to the
         * end-of-buffer state).  Contrast this with the test
         * in input().
         */
        if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            { /* This was really a NUL. */
            yy_state_type yy_next_state;

            yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

            yy_current_state = yy_get_previous_state( yyscanner );

            /* Okay, we're now positioned to make the NUL
             * transition.  We couldn't have
             * yy_get_previous_state() go ahead and do it
             * for us because it doesn't know how to deal
             * with the possibility of jamming (and we don't
             * want to build jamming into it because then it
             * will run more slowly).
             */

            yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

            yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

            if ( yy_next_state )
                {
                /* Consume the NUL. */
                yy_cp = ++yyg->yy_c_buf_p;
                yy_current_state = yy_next_state;
                goto yy_match;
                }

            else
                {
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
                yy_cp = yyg->yy_last_accepting_cpos;
                yy_current_state = yyg->yy_last_accepting_state;
                goto yy_find_action;
                }
            }

        else switch ( yy_get_next_buffer( yyscanner ) )
            {
            case EOB_ACT_END_OF_FILE:
                {
                yyg->yy_did_buffer_switch_on_eof = 0;

                if ( macrowrap(yyscanner ) )
                    {
                    /* Note: because we've taken care in
                     * yy_get_next_buffer() to have set up
                     * yytext, we can now set up
                     * yy_c_buf_p so that if some total
                     * hoser (like flex itself) wants to
                     * call the scanner after we return the
                     * YY_NULL, it'll still work - another
                     * YY_NULL will get returned.
                     */
                    yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

                    yy_act = YY_STATE_EOF(YY_START);
                    goto do_action;
                    }

                else
                    {
                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
                    }
                break;
                }

            case EOB_ACT_CONTINUE_SCAN:
                yyg->yy_c_buf_p =
                    yyg->yytext_ptr + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_match;

            case EOB_ACT_LAST_MATCH:
                yyg->yy_c_buf_p =
                &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

                yy_current_state = yy_get_previous_state( yyscanner );

                yy_cp = yyg->yy_c_buf_p;
                yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
                goto yy_find_action;
            }
        break;
        }

    default:
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--no action found" );
    } /* end of action switch */
        } /* end of scanning one token */
} /* end of macrolex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *  EOB_ACT_LAST_MATCH -
 *  EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *  EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = yyg->yytext_ptr;
    register int number_to_move, i;
    int ret_val;

    if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
        YY_FATAL_ERROR(
        "fatal flex scanner internal error--end of buffer missed" );

    if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
        { /* Don't try to fill the buffer, so this is an EOF. */
        if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
            {
            /* We matched a single character, the EOB, so
             * treat this as a final EOF.
             */
            return EOB_ACT_END_OF_FILE;
            }

        else
            {
            /* We matched some text prior to the EOB, first
             * process it.
             */
            return EOB_ACT_LAST_MATCH;
            }
        }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

    for ( i = 0; i < number_to_move; ++i )
        *(dest++) = *(source++);

    if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
        /* don't do the read, it's not guaranteed to return an EOF,
         * just force an EOF
         */
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

    else
        {
            int num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while ( num_to_read <= 0 )
            { /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

            int yy_c_buf_p_offset =
                (int) (yyg->yy_c_buf_p - b->yy_ch_buf);

            if ( b->yy_is_our_buffer )
                {
                int new_size = b->yy_buf_size * 2;

                if ( new_size <= 0 )
                    b->yy_buf_size += b->yy_buf_size / 8;
                else
                    b->yy_buf_size *= 2;

                b->yy_ch_buf = (char *)
                    /* Include room in for 2 EOB chars. */
                    macrorealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2 ,yyscanner );
                }
            else
                /* Can't grow it, we don't own it. */
                b->yy_ch_buf = 0;

            if ( ! b->yy_ch_buf )
                YY_FATAL_ERROR(
                "fatal error - scanner input buffer overflow" );

            yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                        number_to_move - 1;

            }

        if ( num_to_read > YY_READ_BUF_SIZE )
            num_to_read = YY_READ_BUF_SIZE;

        /* Read in more data. */
        YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
            yyg->yy_n_chars, (size_t) num_to_read );

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    if ( yyg->yy_n_chars == 0 )
        {
        if ( number_to_move == YY_MORE_ADJ )
            {
            ret_val = EOB_ACT_END_OF_FILE;
            macrorestart(yyin  ,yyscanner);
            }

        else
            {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
            }
        }

    else
        ret_val = EOB_ACT_CONTINUE_SCAN;

    if ((yy_size_t) (yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
        /* Extend the array by 50%, plus the number we really need. */
        yy_size_t new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
        YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) macrorealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size ,yyscanner );
        if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
            YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
    }

    yyg->yy_n_chars += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

    yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register yy_state_type yy_current_state;
    register char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
    yy_current_state = yyg->yy_start;

    for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
        {
/* %% [16.0] code to find the next state goes here */
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if ( yy_accept[yy_current_state] )
            {
            yyg->yy_last_accepting_state = yy_current_state;
            yyg->yy_last_accepting_cpos = yy_cp;
            }
        while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
            {
            yy_current_state = (int) yy_def[yy_current_state];
            if ( yy_current_state >= 352 )
                yy_c = yy_meta[(unsigned int) yy_c];
            }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
        }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *  next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    register int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
    register char *yy_cp = yyg->yy_c_buf_p;

    register YY_CHAR yy_c = 1;
    if ( yy_accept[yy_current_state] )
        {
        yyg->yy_last_accepting_state = yy_current_state;
        yyg->yy_last_accepting_cpos = yy_cp;
        }
    while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
        {
        yy_current_state = (int) yy_def[yy_current_state];
        if ( yy_current_state >= 352 )
            yy_c = yy_meta[(unsigned int) yy_c];
        }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    yy_is_jam = (yy_current_state == 351);

    return yy_is_jam ? 0 : yy_current_state;
}

/* %if-c-only */

/* %endif */

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
    int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    *yyg->yy_c_buf_p = yyg->yy_hold_char;

    if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
        {
        /* yy_c_buf_p now points to the character we want to return.
         * If this occurs *before* the EOB characters, then it's a
         * valid NUL; if not, then we've hit the end of the buffer.
         */
        if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
            /* This was really a NUL. */
            *yyg->yy_c_buf_p = '\0';

        else
            { /* need more input */
            int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
            ++yyg->yy_c_buf_p;

            switch ( yy_get_next_buffer( yyscanner ) )
                {
                case EOB_ACT_LAST_MATCH:
                    /* This happens because yy_g_n_b()
                     * sees that we've accumulated a
                     * token and flags that we need to
                     * try matching the token before
                     * proceeding.  But for input(),
                     * there's no matching to consider.
                     * So convert the EOB_ACT_LAST_MATCH
                     * to EOB_ACT_END_OF_FILE.
                     */

                    /* Reset buffer status. */
                    macrorestart(yyin ,yyscanner);

                    /*FALLTHROUGH*/

                case EOB_ACT_END_OF_FILE:
                    {
                    if ( macrowrap(yyscanner ) )
                        return EOF;

                    if ( ! yyg->yy_did_buffer_switch_on_eof )
                        YY_NEW_FILE;
#ifdef __cplusplus
                    return yyinput(yyscanner);
#else
                    return input(yyscanner);
#endif
                    }

                case EOB_ACT_CONTINUE_SCAN:
                    yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
                    break;
                }
            }
        }

    c = *(unsigned char *) yyg->yy_c_buf_p; /* cast for 8-bit char's */
    *yyg->yy_c_buf_p = '\0';    /* preserve yytext */
    yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */
    if ( c == '\n' )

    do{ yylineno++;
        yycolumn=0;
    }while(0)
;

    return c;
}
/* %if-c-only */
#endif  /* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void macrorestart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! YY_CURRENT_BUFFER ){
        macroensure_buffer_stack (yyscanner);
        YY_CURRENT_BUFFER_LVALUE =
            macro_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
    }

    macro_init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
    macro_load_buffer_state(yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* TODO. We should be able to replace this entire function body
     * with
     *      macropop_buffer_state();
     *      macropush_buffer_state(new_buffer);
     */
    macroensure_buffer_stack (yyscanner);
    if ( YY_CURRENT_BUFFER == new_buffer )
        return;

    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    macro_load_buffer_state(yyscanner );

    /* We don't actually know whether we did this switch during
     * EOF (macrowrap()) processing, but the only time this flag
     * is looked at is after macrowrap() is called, so it's safe
     * to go ahead and always set it.
     */
    yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void macro_load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE macro_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE) macroalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in macro_create_buffer()" );

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
     * we need to put in 2 end-of-buffer characters.
     */
    b->yy_ch_buf = (char *) macroalloc(b->yy_buf_size + 2 ,yyscanner );
    if ( ! b->yy_ch_buf )
        YY_FATAL_ERROR( "out of dynamic memory in macro_create_buffer()" );

    b->yy_is_our_buffer = 1;

    macro_init_buffer(b,file ,yyscanner);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with macro_create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if ( ! b )
        return;

    if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

    if ( b->yy_is_our_buffer )
        macrofree((void *) b->yy_ch_buf ,yyscanner );

    macrofree((void *) b ,yyscanner );
}

/* %if-c-only */

/* %endif */

/* %if-c++-only */
/* %endif */

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a macrorestart() or at EOF.
 */
/* %if-c-only */
    static void macro_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
    int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    macro_flush_buffer(b ,yyscanner);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then macro_init_buffer was _probably_
     * called from macrorestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = 0;

/* %endif */
/* %if-c++-only */
/* %endif */
    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void macro_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( ! b )
        return;

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
     * a transition to the end-of-buffer state.  The second causes
     * a jam in that state.
     */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if ( b == YY_CURRENT_BUFFER )
        macro_load_buffer_state(yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void macropush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (new_buffer == NULL)
        return;

    macroensure_buffer_stack(yyscanner);

    /* This block is copied from macro_switch_to_buffer. */
    if ( YY_CURRENT_BUFFER )
        {
        /* Flush out information for old buffer. */
        *yyg->yy_c_buf_p = yyg->yy_hold_char;
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
        }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER)
        yyg->yy_buffer_stack_top++;
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from macro_switch_to_buffer. */
    macro_load_buffer_state(yyscanner );
    yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void macropop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if (!YY_CURRENT_BUFFER)
        return;

    macro_delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if (yyg->yy_buffer_stack_top > 0)
        --yyg->yy_buffer_stack_top;

    if (YY_CURRENT_BUFFER) {
        macro_load_buffer_state(yyscanner );
        yyg->yy_did_buffer_switch_on_eof = 1;
    }
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void macroensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    if (!yyg->yy_buffer_stack) {

        /* First allocation is just for 2 elements, since we don't know if this
         * scanner will even need a stack. We use 2 instead of 1 to avoid an
         * immediate realloc on the next call.
         */
        num_to_alloc = 1;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)macroalloc
                                (num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);
        if ( ! yyg->yy_buffer_stack )
            YY_FATAL_ERROR( "out of dynamic memory in macroensure_buffer_stack()" );

        memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

        yyg->yy_buffer_stack_max = num_to_alloc;
        yyg->yy_buffer_stack_top = 0;
        return;
    }

    if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
        yyg->yy_buffer_stack = (struct yy_buffer_state**)macrorealloc
                                (yyg->yy_buffer_stack,
                                num_to_alloc * sizeof(struct yy_buffer_state*)
                                , yyscanner);
        if ( ! yyg->yy_buffer_stack )
            YY_FATAL_ERROR( "out of dynamic memory in macroensure_buffer_stack()" );

        /* zero only the new slots.*/
        memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
        yyg->yy_buffer_stack_max = num_to_alloc;
    }
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE macro_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;

    if ( size < 2 ||
         base[size-2] != YY_END_OF_BUFFER_CHAR ||
         base[size-1] != YY_END_OF_BUFFER_CHAR )
        /* They forgot to leave room for the EOB's. */
        return 0;

    b = (YY_BUFFER_STATE) macroalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
    if ( ! b )
        YY_FATAL_ERROR( "out of dynamic memory in macro_scan_buffer()" );

    b->yy_buf_size = size - 2;  /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    macro_switch_to_buffer(b ,yyscanner );

    return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to macrolex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       macro_scan_bytes() instead.
 */
YY_BUFFER_STATE macro_scan_string (yyconst char * yystr , yyscan_t yyscanner)
{

    return macro_scan_bytes(yystr,strlen(yystr) ,yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to macrolex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE macro_scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *) macroalloc(n ,yyscanner );
    if ( ! buf )
        YY_FATAL_ERROR( "out of dynamic memory in macro_scan_bytes()" );

    for ( i = 0; i < _yybytes_len; ++i )
        buf[i] = yybytes[i];

    buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

    b = macro_scan_buffer(buf,n ,yyscanner);
    if ( ! b )
        YY_FATAL_ERROR( "bad buffer in macro_scan_bytes()" );

    /* It's okay to grow etc. this buffer, and we should throw it
     * away when we're done.
     */
    b->yy_is_our_buffer = 1;

    return b;
}
/* %endif */

/* %if-c-only */
    static void yy_push_state (int  new_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( yyg->yy_start_stack_ptr >= yyg->yy_start_stack_depth )
        {
        yy_size_t new_size;

        yyg->yy_start_stack_depth += YY_START_STACK_INCR;
        new_size = yyg->yy_start_stack_depth * sizeof( int );

        if ( ! yyg->yy_start_stack )
            yyg->yy_start_stack = (int *) macroalloc(new_size ,yyscanner );

        else
            yyg->yy_start_stack = (int *) macrorealloc((void *) yyg->yy_start_stack,new_size ,yyscanner );

        if ( ! yyg->yy_start_stack )
            YY_FATAL_ERROR( "out of memory expanding start-condition stack" );
        }

    yyg->yy_start_stack[yyg->yy_start_stack_ptr++] = YY_START;

    BEGIN(new_state);
}

/* %if-c-only */
    static void yy_pop_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    if ( --yyg->yy_start_stack_ptr < 0 )
        YY_FATAL_ERROR( "start-condition stack underflow" );

    BEGIN(yyg->yy_start_stack[yyg->yy_start_stack_ptr]);
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
        (void) fprintf( stderr, "%s\n", msg );
    exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
    do \
        { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        yytext[yyleng] = yyg->yy_hold_char; \
        yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
        yyg->yy_hold_char = *yyg->yy_c_buf_p; \
        *yyg->yy_c_buf_p = '\0'; \
        yyleng = yyless_macro_arg; \
        } \
    while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE macroget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int macroget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int macroget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;

    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *macroget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *macroget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int macroget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *macroget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void macroset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void macroset_lineno (int  line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "macroset_lineno called with no buffer" , yyscanner);

    yylineno = line_number;
}

/** Set the current column.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void macroset_column (int  column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "macroset_column called with no buffer" , yyscanner);

    yycolumn = column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see macro_switch_to_buffer
 */
void macroset_in (FILE *  in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = in_str ;
}

void macroset_out (FILE *  out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = out_str ;
}

int macroget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void macroset_debug (int  bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */

YYSTYPE * macroget_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void macroset_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

/* %endif */

/* User-visible API */

/* macrolex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int macrolex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) macroalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* macrolex_init_extra has the same functionality as macrolex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to macroalloc in
 * the yyextra field.
 */

int macrolex_init_extra(YY_EXTRA_TYPE yy_user_defined,yyscan_t* ptr_yy_globals )

{
    struct yyguts_t dummy_yyguts;

    macroset_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) macroalloc ( sizeof( struct yyguts_t ), &dummy_yyguts );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    macroset_extra (yy_user_defined, *ptr_yy_globals);

    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif if-c-only */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from macrolex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = 0;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = (char *) 0;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * macrolex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* macrolex_destroy is for both reentrant and non-reentrant scanners. */
int macrolex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
    while(YY_CURRENT_BUFFER){
        macro_delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
        YY_CURRENT_BUFFER_LVALUE = NULL;
        macropop_buffer_state(yyscanner);
    }

    /* Destroy the stack itself. */
    macrofree(yyg->yy_buffer_stack ,yyscanner);
    yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        macrofree(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * macrolex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    macrofree ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
    register int i;
    for ( i = 0; i < n; ++i )
        s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
    register int n;
    for ( n = 0; s[n]; ++n )
        ;

    return n;
}
#endif

void *macroalloc (yy_size_t  size , yyscan_t yyscanner)
{
    return (void *) malloc( size );
}

void *macrorealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
    /* The cast to (char *) in the following accommodates both
     * implementations that use char* generic pointers, and those
     * that use void* generic pointers.  It works with the latter
     * because both ANSI C and C++ allow castless assignment from
     * any pointer type to void*, and deal with argument conversions
     * as though doing an assignment.
     */
    return (void *) realloc( (char *) ptr, size );
}

void macrofree (void * ptr , yyscan_t yyscanner)
{
    free( (char *) ptr );   /* see macrorealloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 622 "macro.l"




/*

=head1 FUNCTIONS

=over 4

=item C<is_pir_directive>

Returns a non-zero value if the specified name is a PIR directive.

XXX It might be worthwhile to enter the PIR directives as rules; this saves a lot
of checking; these rules should just return TK_ANY.

=cut

*/
PARROT_PURE_FUNCTION
PARROT_WARN_UNUSED_RESULT
static int
is_pir_directive(char const * const name) {

    /* maybe make this a hash or at least a binary search.
     * Or, make these "special" macros, and have them expand
     * to their own spelling. This would remove the need
     * for special code, such as this.
     */
    static char const * const directives[] = {
        ".arg",
        ".begin_call",
        ".begin_return",
        ".begin_yield",
        ".call",
        ".const",
        ".end",
        ".end_call",
        ".end_return",
        ".end_yield",
        ".file",
        ".get_results",
        ".globalconst",
        ".HLL",
        ".HLL_map",
        ".invocant",
        ".lex",
        ".line",
        ".loadlib",
        ".local",
        ".meth_call",
        ".namespace",
        ".nci_call",
        ".param",
        ".return",
        ".sub",
        ".tailcall",
        ".yield",
        NULL /* needed to easily write loops on this array */
    };

    /* iter is a pointer to constant pointers to constant chars. */
    char const * const *iter = directives;

    while (*iter != NULL) {
        if (STREQ(*iter, name))
            return 1;

        iter++;
    }
    return 0;
}

/*

=back

=cut

*/



/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */



